@@ -10,10 +10,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -36,7 +32,7 @@
 static char sccsid[] = "@(#)mkmakefile.c	8.1 (Berkeley) 6/6/93";
 #endif
 static const char rcsid[] =
-  "$FreeBSD: src/usr.sbin/config/mkmakefile.c,v 1.77 2003/06/16 20:08:15 markm Exp $";
+  "$FreeBSD: src/usr.sbin/config/mkmakefile.c,v 1.91 2006/10/24 07:12:31 imp Exp $";
 #endif /* not lint */
 
 /*
@@ -101,8 +97,7 @@ new_fent(void)
 {
 	struct file_list *fp;
 
-	fp = (struct file_list *) malloc(sizeof *fp);
-	bzero(fp, sizeof *fp);
+	fp = (struct file_list *) calloc(1, sizeof *fp);
 	STAILQ_INSERT_TAIL(&ftab, fp, f_next);
 	return (fp);
 }
@@ -117,7 +112,6 @@ makefile(void)
 	char line[BUFSIZ];
 	struct opt *op;
 	int versreq;
-	char *s;
 
 	read_files();
 	snprintf(line, sizeof(line), "../../conf/Makefile.%s", machinename);
@@ -129,12 +123,6 @@ makefile(void)
 	if (ifp == 0)
 		err(1, "%s", line);
 
-	/* XXX this check seems to be misplaced. */
-	if (SLIST_EMPTY(&cputype)) {
-		printf("cpu type must be specified\n");
-		exit(1);
-	}
-
 	ofp = fopen(path("Makefile.new"), "w");
 	if (ofp == 0)
 		err(1, "%s", path("Makefile.new"));
@@ -164,7 +152,8 @@ makefile(void)
 			do_clean(ofp);
 		else if (strncmp(line, "%VERSREQ=", sizeof("%VERSREQ=") - 1) == 0) {
 			versreq = atoi(line + sizeof("%VERSREQ=") - 1);
-			if (versreq != CONFIGVERS) {
+			if (MAJOR_VERS(versreq) != MAJOR_VERS(CONFIGVERS) ||
+			    versreq > CONFIGVERS) {
 				fprintf(stderr, "ERROR: version of config(8) does not match kernel!\n");
 				fprintf(stderr, "config version = %d, ", CONFIGVERS);
 				fprintf(stderr, "version required = %d\n\n", versreq);
@@ -185,15 +174,19 @@ makefile(void)
 	(void) fclose(ifp);
 	(void) fclose(ofp);
 	moveifchanged(path("Makefile.new"), path("Makefile"));
+}
+
+/*
+ * Build hints.c from the skeleton
+ */
+void
+makehints(void)
+{
+	FILE *ifp, *ofp;
+	char line[BUFSIZ];
+	char *s;
+	struct hint *hint;
 
-	/* XXX makefile() should make the Makefile, not hints.c. */
-	if (hints) {
-		ifp = fopen(hints, "r");
-		if (ifp == NULL)
-			err(1, "%s", hints);
-	} else {
-		ifp = NULL;
-	}
 	ofp = fopen(path("hints.c.new"), "w");
 	if (ofp == NULL)
 		err(1, "%s", path("hints.c.new"));
@@ -202,7 +195,10 @@ makefile(void)
 	fprintf(ofp, "\n");
 	fprintf(ofp, "int hintmode = %d;\n", hintmode);
 	fprintf(ofp, "char static_hints[] = {\n");
-	if (ifp) {
+	STAILQ_FOREACH(hint, &hints, hint_next) {
+		ifp = fopen(hint->hint_name, "r");
+		if (ifp == NULL)
+			err(1, "%s", hint->hint_name);
 		while (fgets(line, BUFSIZ, ifp) != 0) {
 			/* zap trailing CR and/or LF */
 			while ((s = rindex(line, '\n')) != NULL)
@@ -232,14 +228,23 @@ makefile(void)
 				continue;
 			fprintf(ofp, "\"%s\\0\"\n", line);
 		}
+		fclose(ifp);
 	}
 	fprintf(ofp, "\"\\0\"\n};\n");
-	if (ifp)
-		fclose(ifp);
 	fclose(ofp);
 	moveifchanged(path("hints.c.new"), path("hints.c"));
+}
+
+/*
+ * Build env.c from the skeleton
+ */
+void
+makeenv(void)
+{
+	FILE *ifp, *ofp;
+	char line[BUFSIZ];
+	char *s;
 
-	/* XXX makefile() should make the Makefile, not env.c. */
 	if (env) {
 		ifp = fopen(env, "r");
 		if (ifp == NULL)
@@ -293,36 +298,26 @@ makefile(void)
 	moveifchanged(path("env.c.new"), path("env.c"));
 }
 
-/*
- * Read in the information about files used in making the system.
- * Store it in the ftab linked list.
- */
 static void
-read_files(void)
+read_file(char *fname)
 {
+	char ifname[MAXPATHLEN];
 	FILE *fp;
-	struct file_list *tp, *pf;
+	struct file_list *tp;
 	struct device *dp;
 	struct opt *op;
-	char *wd, *this, *needs, *compilewith, *depends, *clean, *warning;
-	char fname[MAXPATHLEN];
-	int nreqs, first = 1, isdup, std, filetype,
-	    imp_rule, no_obj, needcount, before_depend, mandatory, nowerror;
+	char *wd, *this, *compilewith, *depends, *clean, *warning;
+	int compile, match, nreqs, std, filetype,
+	    imp_rule, no_obj, before_depend, mandatory, nowerror;
 
-	STAILQ_INIT(&ftab);
-	if (ident == NULL) {
-		printf("no ident line specified\n");
-		exit(1);
-	}
-	(void) snprintf(fname, sizeof(fname), "../../conf/files");
-openit:
 	fp = fopen(fname, "r");
 	if (fp == 0)
 		err(1, "%s", fname);
 next:
 	/*
-	 * filename    [ standard | mandatory | optional | count ]
-	 *	[ dev* | profiling-routine ] [ no-obj ]
+	 * include "filename"
+	 * filename    [ standard | mandatory | optional ]
+	 *	[ dev* [ | dev* ... ] | profiling-routine ] [ no-obj ]
 	 *	[ compile-with "compile rule" [no-implicit-rule] ]
 	 *      [ dependency "dependency-list"] [ before-depend ]
 	 *	[ clean "file-list"] [ warning "text warning" ]
@@ -330,19 +325,8 @@ next:
 	wd = get_word(fp);
 	if (wd == (char *)EOF) {
 		(void) fclose(fp);
-		if (first == 1) {
-			first++;
-			(void) snprintf(fname, sizeof(fname),
-			    "../../conf/files.%s", machinename);
-			fp = fopen(fname, "r");
-			if (fp != 0)
-				goto next;
-			(void) snprintf(fname, sizeof(fname),
-			    "files.%s", machinename);
-			goto openit;
-		}
 		return;
-	}
+	} 
 	if (wd == 0)
 		goto next;
 	if (wd[0] == '#')
@@ -351,6 +335,18 @@ next:
 			;
 		goto next;
 	}
+	if (eq(wd, "include")) {
+		next_quoted_word(fp, wd);
+		if (wd == 0) {
+			printf("%s: missing include filename.\n", fname);
+			exit(1);
+		}
+		(void) snprintf(ifname, sizeof(ifname), "../../%s", wd);
+		read_file(ifname);
+		while (((wd = get_word(fp)) != (char *)EOF) && wd)
+			;
+		goto next;
+	}
 	this = ns(wd);
 	next_word(fp, wd);
 	if (wd == 0) {
@@ -358,21 +354,17 @@ next:
 		    fname, this);
 		exit(1);
 	}
-	if ((pf = fl_lookup(this)) && (pf->f_type != INVISIBLE || pf->f_flags))
-		isdup = ISDUP;
-	else
-		isdup = 0;
-	tp = 0;
+	tp = fl_lookup(this);
+	compile = 0;
+	match = 1;
 	nreqs = 0;
 	compilewith = 0;
 	depends = 0;
 	clean = 0;
 	warning = 0;
-	needs = 0;
 	std = mandatory = 0;
 	imp_rule = 0;
 	no_obj = 0;
-	needcount = 0;
 	before_depend = 0;
 	nowerror = 0;
 	filetype = NORMAL;
@@ -385,19 +377,29 @@ next:
 	 */
 	} else if (eq(wd, "mandatory")) {
 		mandatory = 1;
-	} else if (eq(wd, "count")) {
-		needcount = 1;
 	} else if (!eq(wd, "optional")) {
-		printf("%s: %s must be count, optional, mandatory or standard\n",
+		printf("%s: %s must be optional, mandatory or standard\n",
 		       fname, this);
 		exit(1);
 	}
 nextparam:
 	next_word(fp, wd);
 	if (wd == 0) {
-		if (isdup)
-			goto next;
-		goto doneparam;
+		compile += match;
+		if (compile && tp == NULL)
+			goto doneparam;
+		goto next;
+	}
+	if (eq(wd, "|")) {
+		if (nreqs == 0) {
+			printf("%s: syntax error describing %s\n",
+			    fname, this);
+			exit(1);
+		}
+		compile += match;
+		match = 1;
+		nreqs = 0;
+		goto nextparam;
 	}
 	if (eq(wd, "no-obj")) {
 		no_obj++;
@@ -473,14 +475,9 @@ nextparam:
 		nowerror = 1;
 		goto nextparam;
 	}
-	if (needs == 0 && nreqs == 1)
-		needs = ns(wd);
-	if (isdup)
-		goto invis;
 	STAILQ_FOREACH(dp, &dtab, d_next)
 		if (eq(dp->d_name, wd)) {
-			if (std && dp->d_count <= 0)
-				dp->d_count = 1;
+			dp->d_done |= DEVDONE;
 			goto nextparam;
 		}
 	if (mandatory) {
@@ -494,29 +491,10 @@ nextparam:
 		exit(1);
 	}
 	SLIST_FOREACH(op, &opt, op_next)
-		if (op->op_value == 0 && opteq(op->op_name, wd)) {
-			if (nreqs == 1) {
-				free(needs);
-				needs = 0;
-			}
+		if (op->op_value == 0 && opteq(op->op_name, wd))
 			goto nextparam;
-		}
-invis:
-	while ((wd = get_word(fp)) != 0)
-		;
-	if (tp == 0)
-		tp = new_fent();
-	tp->f_fn = this;
-	tp->f_type = INVISIBLE;
-	tp->f_needs = needs;
-	tp->f_flags |= isdup;
-	if (needcount)
-		tp->f_flags |= NEED_COUNT;
-	tp->f_compilewith = compilewith;
-	tp->f_depends = depends;
-	tp->f_clean = clean;
-	tp->f_warn = warning;
-	goto next;
+	match = 0;
+	goto nextparam;
 
 doneparam:
 	if (std == 0 && nreqs == 0) {
@@ -532,31 +510,47 @@ doneparam:
 	}
 	if (filetype == PROFILING && profiling == 0)
 		goto next;
-	if (tp == 0)
-		tp = new_fent();
+	tp = new_fent();
 	tp->f_fn = this;
 	tp->f_type = filetype;
-	tp->f_flags &= ~ISDUP;
 	if (imp_rule)
 		tp->f_flags |= NO_IMPLCT_RULE;
 	if (no_obj)
 		tp->f_flags |= NO_OBJ;
 	if (before_depend)
 		tp->f_flags |= BEFORE_DEPEND;
-	if (needcount)
-		tp->f_flags |= NEED_COUNT;
 	if (nowerror)
 		tp->f_flags |= NOWERROR;
-	tp->f_needs = needs;
 	tp->f_compilewith = compilewith;
 	tp->f_depends = depends;
 	tp->f_clean = clean;
 	tp->f_warn = warning;
-	if (pf && pf->f_type == INVISIBLE)
-		pf->f_flags |= ISDUP;		/* mark as duplicate */
 	goto next;
 }
 
+/*
+ * Read in the information about files used in making the system.
+ * Store it in the ftab linked list.
+ */
+static void
+read_files(void)
+{
+	char fname[MAXPATHLEN];
+	struct files_name *nl, *tnl;
+	
+	(void) snprintf(fname, sizeof(fname), "../../conf/files");
+	read_file(fname);
+	(void) snprintf(fname, sizeof(fname),
+		       	"../../conf/files.%s", machinename);
+	read_file(fname);
+	for (nl = STAILQ_FIRST(&fntab); nl != NULL; nl = tnl) {
+		read_file(nl->f_name);
+		tnl = STAILQ_NEXT(nl, f_next);
+		free(nl->f_name);
+		free(nl);
+	}
+}
+
 static int
 opteq(const char *cp, const char *dp)
 {
@@ -609,7 +603,7 @@ do_objs(FILE *fp)
 	fprintf(fp, "OBJS=");
 	lpos = 6;
 	STAILQ_FOREACH(tp, &ftab, f_next) {
-		if (tp->f_type == INVISIBLE || tp->f_flags & NO_OBJ)
+		if (tp->f_flags & NO_OBJ)
 			continue;
 		sp = tail(tp->f_fn);
 		cp = sp + (len = strlen(sp)) - 1;
@@ -645,7 +639,7 @@ do_xxfiles(char *tag, FILE *fp)
 	fprintf(fp, "%sFILES=", SUFF);
 	lpos = 8;
 	STAILQ_FOREACH(tp, &ftab, f_next)
-		if (tp->f_type != INVISIBLE && tp->f_type != NODEPEND) {
+		if (tp->f_type != NODEPEND) {
 			len = strlen(tp->f_fn);
 			if (tp->f_fn[len - slen - 1] != '.')
 				continue;
@@ -683,13 +677,11 @@ tail(char *fn)
 static void
 do_rules(FILE *f)
 {
-	char *cp, *np, och, *tp;
+	char *cp, *np, och;
 	struct file_list *ftp;
 	char *compilewith;
 
 	STAILQ_FOREACH(ftp, &ftab, f_next) {
-		if (ftp->f_type == INVISIBLE)
-			continue;
 		if (ftp->f_warn)
 			printf("WARNING: %s\n", ftp->f_warn);
 		cp = (np = ftp->f_fn) + strlen(ftp->f_fn) - 1;
@@ -722,7 +714,6 @@ do_rules(FILE *f)
 					np, och);
 			}
 		}
-		tp = tail(np);
 		compilewith = ftp->f_compilewith;
 		if (compilewith == 0) {
 			const char *ftype = NULL;
