@@ -5,7 +5,7 @@
 #
 # Please see src/share/examples/etc/bsd-style-copyright.
 #
-# $FreeBSD: src/usr.sbin/mtree/test/test03.sh,v 1.1 2003/11/05 22:26:39 phk Exp $
+# $FreeBSD: src/usr.sbin/mtree/test/test03.sh,v 1.2 2005/03/29 11:44:17 tobez Exp $
 #
 
 set -e
@@ -15,7 +15,7 @@ TMP=/tmp/mtree.$$
 rm -rf ${TMP}
 mkdir -p ${TMP}
 
-K=uid,uname,gid,gname,flags,md5digest,size,ripemd160digest,sha1digest,cksum
+K=uid,uname,gid,gname,flags,md5digest,size,ripemd160digest,sha1digest,sha256digest,cksum
 
 rm -rf _FOO
 mkdir _FOO
